package src;

import src.DrawningObjects.DrawningAirplane;
import src.Generics.AirplaneGenericCollection;
import src.MovementStrategy.DrawningObjectAirplane;
import src.Generics.AirplaneGenericStorage;
import src.Generics.AirplaneTrashCollection;
import src.Exceptions.AirplaneNotFoundException;
import src.Exceptions.AirplaneStorageOverflowException;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import javax.swing.filechooser.FileFilter;
import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;

import org.apache.logging.log4j.*;


class TxtSaveFilter extends FileFilter {
    @Override
    public boolean accept(File f) {
        if (f.isDirectory()) {
            return false;
        }
        String s = f.getName().toLowerCase();
        return s.endsWith(".txt");
    }
    @Override
    public String getDescription() {
        return "*.txt";
    }
}

public class FormAirplaneCollection {
    private final Logger _logger;
    private final AirplaneGenericStorage _storage;
    private JList<String> listBoxStorages;
    private DefaultListModel<String> listBoxModel;
    private int pictureBoxWidth = 630;
    private int pictureBoxHeight = 426;
    CollectionCanvas canv;

    void Draw() {
        if (canv == null)
            return;
        canv.repaint();
    }

    private void ReloadObjects() {
        int index = listBoxStorages.getSelectedIndex();
        listBoxModel.clear();
        List<String> keys = _storage.Keys();
        for (int i = 0; i < keys.size(); i++) {
            listBoxModel.addElement(keys.get(i));
        }
        if (listBoxModel.size() > 0 && (index == -1 || index >= listBoxModel.size()))
            listBoxStorages.setSelectedIndex(0);
        else if (listBoxModel.size() > 0)
            listBoxStorages.setSelectedIndex(index);
    }

    public FormAirplaneCollection() {
        _logger = LogManager.getLogger("logger");
        JMenuBar menuFile = new JMenuBar();
        JMenu file = new JMenu("Файл");
        menuFile.add(file);
        JMenuItem saveFile = new JMenuItem("Сохранить");
        JMenuItem loadFile = new JMenuItem("Загрузить");
        JMenuItem saveCollection = new JMenuItem("Сохранить коллекцию");
        JMenuItem loadCollection = new JMenuItem("Загрузить коллекцию");
        file.add(saveCollection);
        file.add(loadCollection);
        file.add(saveFile);
        file.add(loadFile);
        AirplaneTrashCollection<DrawningAirplane> _trashCollection = new AirplaneTrashCollection<>();
        JButton callTrashButton = new JButton("Мусор");
        _storage = new AirplaneGenericStorage(pictureBoxWidth, pictureBoxHeight);
        JScrollPane scrollPane = new JScrollPane();
        canv = new CollectionCanvas();
        JPanel toolBox = new JPanel();
        JTextField storageName = new JTextField();
        JButton addStorageButton = new JButton("Добавить набор");
        listBoxModel = new DefaultListModel<>();
        listBoxStorages = new JList<>(listBoxModel);
        scrollPane.setViewportView(listBoxStorages);
        JButton delStorageButton = new JButton("Удалить набор");
        JFrame collectionFrame = new JFrame();
        collectionFrame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        collectionFrame.setSize(880, 497);
        toolBox.setBounds(623, 12, 227, 426);
        canv.setBounds(12, 12, pictureBoxWidth, pictureBoxHeight);
        JButton addButton = new JButton("Добавить");
        JButton removeButton = new JButton("Удалить");
        JButton refreshButton = new JButton("Обновить");
        JTextField airplaneNumb = new JTextField();
        GridLayout lay = new GridLayout(9, 1);
        toolBox.add(storageName);
        toolBox.add(addStorageButton);
        toolBox.add(scrollPane);
        toolBox.add(delStorageButton);
        toolBox.setLayout(lay);
        toolBox.add(addButton);
        toolBox.add(airplaneNumb);
        toolBox.add(removeButton);
        toolBox.add(callTrashButton);
        toolBox.add(refreshButton);
        collectionFrame.add(toolBox);
        collectionFrame.setJMenuBar(menuFile);
        collectionFrame.add(canv);
        collectionFrame.setVisible(true);
        canv._storage = _storage;
        canv.listBoxStorages = listBoxStorages;
        saveFile.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser("C:\\Users\\1\\Desktop\\свое");
                fc.addChoosableFileFilter(new TxtSaveFilter());
                int retrieval = fc.showSaveDialog(null);
                if (retrieval == JFileChooser.APPROVE_OPTION) {
                    File file = new File(fc.getSelectedFile() + "." + "txt");
                    try {
                        _storage.SaveData(file);
                        _logger.info("Сохранено в файл: " + file.getAbsolutePath());
                    } catch (Exception ex) {
                        JOptionPane.showMessageDialog(null, "Ошибка при сохранении всех объектов " + ex.getMessage(), "Результат", JOptionPane.ERROR_MESSAGE);
                        _logger.error("Не удалось сохранить объекты в файл: " + file.getAbsolutePath());
                    }
                }
            }
        });
        saveCollection.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser("C:\\Users\\1\\Desktop\\свое");
                fc.addChoosableFileFilter(new TxtSaveFilter());
                int retrieval = fc.showSaveDialog(null);
                if (retrieval == JFileChooser.APPROVE_OPTION) {
                    File file = new File(fc.getSelectedFile() + "." + "txt");
                    try {
                        if(listBoxStorages.getSelectedIndex() == -1) {
                            return;
                        }
                        _storage._airplaneStorages.get(listBoxStorages.getSelectedValue()).SaveData(file, listBoxStorages.getSelectedValue());
                        ReloadObjects();
                        _logger.info("Сохранена коллекция в файл: " + file.getAbsolutePath());
                    } catch (Exception ex) {
                        JOptionPane.showMessageDialog(null, "Ошибка при сохранении коллекции " + ex.getMessage(), "Результат", JOptionPane.ERROR_MESSAGE);
                        _logger.error("Не удалось сохранить коллекцию в файл: " + file.getAbsolutePath());
                    }
                }
            }
        });
        loadFile.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser("C:\\Users\\1\\Desktop\\свое");
                int ret = fc.showDialog(null, "Открыть файл");
                if(ret == JFileChooser.APPROVE_OPTION){
                    File file = fc.getSelectedFile();
                    try {
                        _storage.LoadData(file);
                        canv._storage =_storage;
                        _logger.info("Загружено из файла: " + file.getAbsolutePath());
                    } catch (FileNotFoundException ex) {
                        JOptionPane.showMessageDialog(null, "Ошибка при загрузке всех объектов " + ex.getMessage(), "Результат", JOptionPane.ERROR_MESSAGE);
                        _logger.error("Не удалось загрузить объекты из файла: " + file.getAbsolutePath());
                    }
                    catch (Exception ex){
                        _logger.fatal("фатальная ошибка");
                    }
                }
                ReloadObjects();
                canv.repaint();
            }
        });
        loadCollection.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser("C:\\Users\\1\\Desktop\\свое");
                int ret = fc.showDialog(null, "Открыть файл");
                if(ret == JFileChooser.APPROVE_OPTION){
                    File file = fc.getSelectedFile();
                    try {
                        _storage.LoadCollection(file);
                        ReloadObjects();
                        canv.repaint();
                        _logger.info("Загружен файл с коллекцией: " + file.getAbsolutePath());
                    } catch (IOException ex) {
                        JOptionPane.showMessageDialog(null, "Ошибка при загрузке коллекции " + ex.getMessage(), "Результат", JOptionPane.ERROR_MESSAGE);
                        _logger.error("Не удалось загрузить коллекцию из файла: " + file.getAbsolutePath());
                    }
                }
            }
        });
        addStorageButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (storageName.getText() == null)
                    _logger.error("Не удалось добавить набор: " + storageName);
                _storage.AddSet(storageName.getText());
                _logger.info("Добавлен набор: " + storageName.getText());
                ReloadObjects();
            }
        });
        delStorageButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (listBoxStorages.getSelectedIndex() == -1) {
                    _logger.error("Не выбран набор для удаления");
                }
                _storage.DelSet(listBoxStorages.getSelectedValue(), _trashCollection);
                ReloadObjects();
                _logger.info("Удален набор: " + listBoxStorages.getSelectedValue());
            }
        });
        callTrashButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_trashCollection.GetSize() == 0)
                    return;
                FormAirplaneWithRadar form = new FormAirplaneWithRadar();
                form.ChangeAirplane(_trashCollection.GetTop());
                _trashCollection.Pop();
            }
        });
        addButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (listBoxStorages.getSelectedIndex() == -1) {
                    _logger.warn("Не выбрана коллекция для добавления объекта");
                }
                AirplaneGenericCollection<DrawningAirplane, DrawningObjectAirplane> _airplanes = _storage.Get(listBoxStorages.getSelectedValue());
                FormAirplaneConfig form = new FormAirplaneConfig();
                form.addButton.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        try {
                            if (_airplanes.Insert(form._airplane)) {
                                JOptionPane.showMessageDialog(null, "Объект добавлен", "Информация", JOptionPane.INFORMATION_MESSAGE);
                                form._airplane._pictureWidth = pictureBoxWidth;
                                form._airplane._pictureHeight = pictureBoxHeight;
                                Draw();
                            } else {
                                JOptionPane.showMessageDialog(null, "Не удалось добавить объект", "Информация", JOptionPane.INFORMATION_MESSAGE);
                                _logger.warn("Не удалось добавить объект " );
                            }
                            form.frameConfig.dispose();
                            Draw();
                            _logger.info("Добавлен объект");
                        } catch (AirplaneStorageOverflowException ex) {
                            JOptionPane.showMessageDialog(null, ex.getMessage());
                            _logger.warn("Не удалось добавить объект " + ex.getMessage());
                        }
                        catch (Exception ex){
                            _logger.fatal("фатальная ошибка");
                        }
                    }
                });
                form.cancelButton.addActionListener(new ActionListener() {
                    @Override
                    public void actionPerformed(ActionEvent e) {
                        form.frameConfig.dispose();
                        Draw();
                    }
                });
            }
        });
        removeButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (listBoxStorages.getSelectedIndex() == -1) {
                    _logger.warn("Не выбрана коллекция для удаления объекта");
                }
                AirplaneGenericCollection<DrawningAirplane, DrawningObjectAirplane> _airplanes = _storage.Get(listBoxStorages.getSelectedValue());
                if (_airplanes == null) {
                    _logger.warn("Набор равен null");
                }
                String tmp = airplaneNumb.getText();
                int numb=Integer.parseInt(tmp);
                try {
                    if (_airplanes.Get(numb)!=null){
                        DrawningAirplane curAirplane = _airplanes.Get(numb);
                        _trashCollection.Push(curAirplane);
                        _airplanes.Remove(numb);
                        _airplanes.ShowAirplanes();
                        JOptionPane.showMessageDialog(null, "Объект удален", "Информация", JOptionPane.INFORMATION_MESSAGE);
                        Draw();
                        _logger.info("Удален объект по позиции " + numb);
                    }     else{
                        throw new AirplaneNotFoundException(numb);
                    }

                } catch (AirplaneNotFoundException ex) {
                    JOptionPane.showMessageDialog(null, ex.getMessage());
                    _logger.warn("Не удалось удалить объект" + ex.getMessage());
                }
                catch (Exception ex){
                    _logger.fatal("фатальная ошибка");
                }
            }
        });

        refreshButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (listBoxStorages.getSelectedIndex() == -1) {
                    return;
                }
                AirplaneGenericCollection<DrawningAirplane, DrawningObjectAirplane> _airplanes = _storage.Get(listBoxStorages.getSelectedValue());
                if (_airplanes == null) {
                    return;
                }
                _airplanes.ShowAirplanes();
                Draw();
            }
        });
    }

    class CollectionCanvas extends JComponent {
        public AirplaneGenericStorage _storage;
        public JList<String> listBoxStorages;

        public CollectionCanvas() {
        }

        @Override
        public void paintComponent(Graphics g) {
            if (listBoxStorages == null || listBoxStorages.getSelectedIndex() == -1) {
                return;
            }
            super.paintComponents(g);
            Graphics2D g2d = (Graphics2D) g;
            g2d.drawImage(_storage.Get(listBoxStorages.getSelectedValue()).ShowAirplanes(), 0, 0, this);
            super.repaint();
        }
    }
}
