package src.Generics;
import src.DrawningObjects.DrawningAirplane;
import src.MovementStrategy.IMoveableObject;
import src.DrawningObjects.ExtentionDrawningAirplane;


import com.sun.nio.sctp.InvalidStreamException;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

public class AirplaneGenericCollection<T extends DrawningAirplane, U extends IMoveableObject> {
    public static char _separatorRecords = ';';
    public static char _separatorForObject = ':';
    private final int _pictureWidth;
    private final int _pictureHeight;
    private final int _placeSizeWidth = 210;
    private final int _placeSizeHeight = 78;
    private SetGeneric<T> _collection;
    public ArrayList<T> GetAirplanes(){
        return _collection.GetAirplanes(_collection.Count);
    }

    public boolean SaveData(File f, String name) throws IOException {
        if(f.exists()) {
            f.delete();
        }
        f.createNewFile();
        StringBuilder data = new StringBuilder();
        data.append("AirplaneCollection\n");
        data.append(String.format("%s\n", name));
        StringBuilder records = new StringBuilder();
        for(DrawningAirplane elem : GetAirplanes())
        {
            records.append(String.format("%s%c", ExtentionDrawningAirplane.GetDataForSave(elem, _separatorForObject), _separatorRecords));
        }
        data.append(records);
        if(data.length() == 0)
            throw new InvalidStreamException("Нет данных для сохранения");
        FileWriter writer = new FileWriter(f);
        writer.write(data.toString());
        writer.flush();
        writer.close();
        return true;
    }
    public AirplaneGenericCollection(int picWidth, int picHeight){
        int width = picWidth / _placeSizeWidth;
        int height = picHeight / _placeSizeHeight;
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _collection = new SetGeneric<T>(width * height);
    }
    public int Size(){
        return _collection.Count;
    }
    public boolean Insert(T obj){
        if (obj == null)
        {
            return false;
        }
        return _collection.Insert(obj);
    }
    public boolean Remove(int position){
        return _collection.Remove(position);
    }
    public T Get(int position){
        if(position < 0 || position >= _collection.Count)
            return null;
        return _collection.Get(position);
    }
    public U GetU(int pos){
        T ans = _collection.Get(pos);
        if(ans == null)
            return null;
        return (U)ans.GetMoveableObject();
    }
    private void DrawBackground(Graphics g)
    {
        g.setColor(Color.BLACK);
        for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
        {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                    1; ++j)
            {
                g.drawLine(i * _placeSizeWidth, j *
                        _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                        _placeSizeHeight);
            }
            g.drawLine(i * _placeSizeWidth, 0, i *
                    _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
        }
    }
    private void DrawObjects(Graphics g)
    {
        for (int i = 0; i < _collection.Count; i++)
        {
            DrawningAirplane airplane = _collection.Get(i);
            if (airplane != null)
            {
                int inRow = _pictureWidth / _placeSizeWidth;
                airplane.SetPosition(_pictureWidth - _placeSizeWidth - (i % inRow * _placeSizeWidth), i / inRow * _placeSizeHeight );
                airplane.DrawAirplane((Graphics2D) g);
            }
        }
    }
    public BufferedImage ShowAirplanes()
    {
        BufferedImage bmp = new BufferedImage(_pictureWidth, _pictureHeight, BufferedImage.TYPE_4BYTE_ABGR);
        Graphics gr = bmp.createGraphics();
        DrawBackground(gr);
        DrawObjects(gr);
        return bmp;
    }
    public void Clear(){
        _collection = new SetGeneric<>(_pictureWidth * _pictureHeight);
    }
}
